!***************************************************************************
!*   Copyright (C) 2012 by Graziano Giuliani                               *
!*   ggiulian at ictp.it                                                   *
!*                                                                         *
!*   This program is free software; you can redistribute it and/or modify  *
!*   it under the terms of the GNU General Public License as published by  *
!*   the Free Software Foundation; either version 2 of the License, or     *
!*   (at your option) any later version.                                   *
!*                                                                         *
!*   This program is distributed in the hope that it will be useful,       *
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
!*   GNU General Public License for more details. (see COPYING)            *
!*                                                                         *
!*   You should have received a copy of the GNU General Public License     *
!*   along with this program; if not, write to the                         *
!*   Free Software Foundation, Inc.,                                       *
!*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
!*                                                                         *
!*   LIC: GPL                                                              *
!*                                                                         *
!***************************************************************************

program example_netcdf
  use netcdf
  implicit none
  integer , parameter :: dp = kind(1.0D0)
  real(dp) :: x , y
  real(dp) , dimension(:,:) , allocatable :: matrix
  real(dp) , dimension(2) :: dxy
  integer :: i , j , it , hx , nx , nt
  integer , dimension(3) :: idims
  integer , dimension(3) :: istart , icount
  integer :: errval , ncid , ivar
  integer , dimension(8) :: xtime
  character(len=256) :: prgname , filename , argread , history
  integer :: argc

  x = 0.0
  y = 0.0

  call get_command_argument(0, prgname)
  argc = command_argument_count()
  if ( argc < 3 ) then
    print *, "Not enough arguments."
    print *, "Usage:"
    print *, trim(prgname), " nx nt outfile"
    print *, "where: "
    print *, "      nx is the size of the problem"
    print *, "      nt is the number of timesteps"
    print *, "      outfile is the name of the output file"
    stop
  end if
  
  call get_command_argument(1, argread)
  read(argread, "(i8)") nx
  call get_command_argument(2, argread)
  read(argread, "(i8)") nt
  if ( nx < 1 .or. nt < 1 ) then
    print *, " NX and NT must be greater than 0"
    stop
  end if
  hx = nx/2
  call get_command_argument(3, filename)

  print *, "NX = ", nx
  print *, "NT = ", nt

  errval = nf90_create(filename, NF90_NOCLOBBER, ncid)
  if (errval /= NF90_NOERR) then
    print *, "Cannot write output file ", trim(filename)
    print *, nf90_strerror(errval)
    stop
  end if

  errval = nf90_put_att(ncid, nf90_global, "title", &
                        "Random walk of a log surface")
  if (errval /= NF90_NOERR) then
    print *, "Cannot add title"
    print *, nf90_strerror(errval)
    stop
  end if

  call date_and_time(values=xtime)
  write (history,'(a,i0.4,a,i0.2,a,i0.2,a,i0.2,a,i0.2,a,i0.2,a)')   &
             "Created on ", xtime(1) , '-' , xtime(2) , '-' , xtime(3) , &
             ' ' , xtime(5) , ':' , xtime(6) , ':' , xtime(7)
  errval = nf90_put_att(ncid, nf90_global, "history", history)
  if (errval /= NF90_NOERR) then
    print *, "Cannot add history"
    print *, nf90_strerror(errval)
    stop
  end if

  errval = nf90_put_att(ncid, nf90_global, "purpose", &
                        "Test Fortran90 netCDF interface")
  if (errval /= NF90_NOERR) then
    print *, "Cannot add purpose"
    print *, nf90_strerror(errval)
    stop
  end if

  errval = nf90_def_dim(ncid, "X", nx, idims(1))
  if (errval /= NF90_NOERR) then
    print *, "Cannot define dimension X"
    print *, nf90_strerror(errval)
    stop
  end if

  errval = nf90_def_dim(ncid, "Y", nx, idims(2))
  if (errval /= NF90_NOERR) then
    print *, "Cannot define dimension Y"
    print *, nf90_strerror(errval)
    stop
  end if

  errval = nf90_def_dim(ncid, "time", nf90_unlimited, idims(3))
  if (errval /= NF90_NOERR) then
    print *, "Cannot define dimension time"
    print *, nf90_strerror(errval)
    stop
  end if

  errval = nf90_def_var(ncid, "randomwalk", nf90_double, idims, ivar)
  if (errval /= NF90_NOERR) then
    print *, "Cannot define variable randomwalk"
    print *, nf90_strerror(errval)
    stop
  end if

  errval = nf90_put_att(ncid, ivar, "units", "dimensionless")
  if (errval /= NF90_NOERR) then
    print *, "Cannot add attribute units"
    print *, nf90_strerror(errval)
    stop
  end if

  errval = nf90_enddef(ncid)
  if (errval /= NF90_NOERR) then
    print *, "Cannot Exit Define Mode"
    print *, nf90_strerror(errval)
    stop
  end if

  allocate(matrix(nx,nx))

  call random_seed

  do it = 1 , nt
    call random_number(dxy)
    x = x + dxy(1)
    y = y + dxy(2)
    do j = 1 , nx
      do i = 1 , nx
        matrix(i,j) = dlog((dble(i-hx)-x)**2+(dble(j-hx)-y)**2+0.1)
      end do
    end do
    istart(1) = it
    istart(2) = 1
    istart(3) = 1
    icount(1) = 1
    icount(2) = nx
    icount(3) = nx
    errval = nf90_put_var(ncid, ivar, matrix, istart, icount)
    if (errval /= NF90_NOERR) then
      print *, "Cannot write step ", it
      print *, nf90_strerror(errval)
      stop
    end if
    errval = nf90_sync(ncid)
    if (errval /= NF90_NOERR) then
      print *, "Cannot sync file"
      print *, nf90_strerror(errval)
      stop
    end if
  end do

  errval = nf90_close(ncid)
  if (errval /= NF90_NOERR) then
    print *, "Cannot close file"
    print *, nf90_strerror(errval)
    stop
  end if
  
  deallocate(matrix)
  stop
end program example_netcdf
