program numII_advection
  use iso_fortran_env
  use netcdf
  implicit none

  character(len=64) :: progname , argument , first_part

  real(8) , parameter :: x0 = 0.0D0
  real(8) , parameter :: t0 = 0.0D0
  real(8) , parameter :: x1 = 1000.0D0
  real(8) , parameter :: dx = 1.0D0
  real(8) , parameter :: u =  0.250D0
  integer , parameter :: ishape = 2

  real(8) , allocatable , dimension(:) :: x
  real(8) , allocatable , dimension(:) :: d
  real(8) , allocatable , dimension(:) :: y0
  real(8) , allocatable , dimension(:) :: yn
  real(8) , allocatable , dimension(:) :: ynp1
  real(8) , allocatable , dimension(:) :: ynm1

  real(8) , parameter :: xmax_ftfs = 10.0D0
  real(8) , parameter :: xmax_ftbs = 8000.0D0
  real(8) , parameter :: xmax_ctcs = 8000.0D0
  real(8) , parameter :: print_step_ftfs = 10.0D0
  real(8) , parameter :: print_step_ftbs = 100.0D0
  real(8) , parameter :: print_step_ctcs = 100.0D0

  ! Input command line
  real(8) :: dt
  real(8) :: alpha
  real(8) :: beta

  real(8) :: time
  integer :: it , nt , nj , ierr
  integer , dimension(5) :: nchandlers

  if ( command_argument_count() < 1 ) stop
  call get_command_argument(0,value=progname)
  call get_command_argument(1,value=argument)
  read(argument,*) dt,alpha,beta

  nj = int(( x1 - x0 ) / dx) + 1
  nt = int(( xmax_ftfs/u - t0 ) / dt)

  allocate(x(nj),y0(nj),yn(nj),ynp1(nj),ynm1(nj),d(nj),stat=ierr)
  if ( ierr /= 0 ) then
    write(error_unit) 'Error in allocation !'
    stop
  end if

  call xaxis(x,x0,dx)
  call phi(x,y0,ishape)

  first_part = trim(progname)//'_FTFS'
  yn = y0
  call open_outfile(first_part,u,x,nchandlers,'Advection using FTFS scheme')
  call write_solution(nchandlers,0.0D0,yn)

  do it = 1 , nt
    time = dt * dble(it)
    call FTFS(ynp1,yn,u,dx,dt)
    if ( mod(time,print_step_ftfs) < dt/2.0D0 ) then
      call write_solution(nchandlers,time,ynp1)
    end if
    yn = ynp1
  end do
  call close_outfile(nchandlers)

  write(output_unit,*) 'FTBS BLOWS UP                   : ', &
          sum(abs(ynp1-y0))/dble(nj)

  nt = int(( xmax_ctcs/u - t0 ) / dt)
  yn = y0

  first_part = trim(progname)//'_CTCS'
  call open_outfile(first_part,u,x,nchandlers,'Advection using CTCS scheme')
  call write_solution(nchandlers,0.0D0,yn)

  call FTBS(ynp1,yn,u,dx,dt)
  ynm1 = yn
  yn = ynp1

  do it = 2 , nt
    time = dt * dble(it)
    call CTCS(ynp1,yn,ynm1,u,dx,dt)
    if ( mod(time,print_step_ctcs) < dt/2.0D0 ) then
      call write_solution(nchandlers,time,ynp1)
    end if
    ynm1 = yn
    yn = ynp1
  end do
  nt = int(( xmax_ftbs/u - t0 ) / dt)
  yn = y0

  first_part = trim(progname)//'_FTBS'
  call open_outfile(first_part,u,x,nchandlers,'Advection using FTBS scheme')
  call write_solution(nchandlers,0.0D0,yn)

  do it = 1 , nt
    time = dt * dble(it)
    call FTBS(ynp1,yn,u,dx,dt)
    if ( mod(time,print_step_ftbs) < dt/2.0D0 ) then
      call write_solution(nchandlers,time,ynp1)
    end if
    yn = ynp1
  end do
  call close_outfile(nchandlers)

  write(output_unit,*) 'FTBS Mean Error                 : ', &
          sum(abs(ynp1-y0))/dble(nj)

  nt = int(( xmax_ctcs/u - t0 ) / dt)
  yn = y0

  first_part = trim(progname)//'_CTCS'
  call open_outfile(first_part,u,x,nchandlers,'Advection using CTCS scheme')
  call write_solution(nchandlers,0.0D0,yn)

  call FTBS(ynp1,yn,u,dx,dt)
  ynm1 = yn
  yn = ynp1

  do it = 2 , nt
    time = dt * dble(it)
    call CTCS(ynp1,yn,ynm1,u,dx,dt)
    if ( mod(time,print_step_ctcs) < dt/2.0D0 ) then
      call write_solution(nchandlers,time,ynp1)
    end if
    ynm1 = yn
    yn = ynp1
  end do
  call close_outfile(nchandlers)

  write(output_unit,*) 'CTCS Mean Error                 : ', &
          sum(abs(ynp1-y0))/dble(nj)

  yn = y0
  first_part = trim(progname)//'_CTCS_RA'
  call open_outfile(first_part,u,x,nchandlers, &
    'Advection using CTFS scheme with Robert-Asselin filter',alpha)
  call write_solution(nchandlers,0.0D0,yn)

  call FTBS(ynp1,yn,u,dx,dt)
  ynm1 = yn
  yn = ynp1

  do it = 2 , nt
    time = dt * dble(it)
    call CTCS(ynp1,yn,ynm1,u,dx,dt)
    if ( mod(time,print_step_ctcs) < dt/2.0D0 ) then
      call write_solution(nchandlers,time,ynp1)
    end if
    call RA_filter_update(alpha,yn,ynm1,ynp1,d)
    ynm1 = yn + d
    yn = ynp1
  end do
  call close_outfile(nchandlers)

  write(output_unit,*) 'CTCS with RA filter Mean Error  : ', &
          sum(abs(ynp1-y0))/dble(nj)

  yn = y0
  first_part = trim(progname)//'_CTCS_RAW'
  call open_outfile(first_part,u,x,nchandlers, &
    'Advection using CTFS scheme with Robert-Asselin-William filter',alpha,beta)
  call write_solution(nchandlers,0.0D0,yn)

  call FTBS(ynp1,yn,u,dx,dt)
  ynm1 = yn
  yn = ynp1

  do it = 2 , nt
    time = dt * dble(it)
    call CTCS(ynp1,yn,ynm1,u,dx,dt)
    if ( mod(time,print_step_ctcs) < dt/2.0D0 ) then
      call write_solution(nchandlers,time,ynp1)
    end if
    call RA_filter_update(alpha,yn,ynm1,ynp1,d)
    ynm1 = yn + beta*d
    yn = ynp1 + (beta-1.0D0)*d
  end do
  call close_outfile(nchandlers)

  write(output_unit,*) 'CTCS with RAW filter Mean Error : ', &
          sum(abs(ynp1-y0))/dble(nj)

  yn = y0
  first_part = trim(progname)//'_SLLN'
  call open_outfile(first_part,u,x,nchandlers, &
    'Advection using SL scheme with linear interpolation')
  call write_solution(nchandlers,0.0D0,yn)

  do it = 1 , nt
    time = dt * dble(it)
    call SLAD_LIN(x,ynp1,yn,u,dx,dt)
    if ( mod(time,print_step_ftbs) < dt/2.0D0 ) then
      call write_solution(nchandlers,time,ynp1)
    end if
    yn = ynp1
  end do
  call close_outfile(nchandlers)

  write(output_unit,*) 'SL Linear Mean Error            : ', &
          sum(abs(ynp1-y0))/dble(nj)

  yn = y0
  first_part = trim(progname)//'_SLCB'
  call open_outfile(first_part,u,x,nchandlers, &
    'Advection using SL scheme with cubic interpolation')
  call write_solution(nchandlers,0.0D0,yn)

  do it = 1 , nt
    time = dt * dble(it)
    call SLAD_CUB(x,ynp1,yn,u,dx,dt)
    if ( mod(time,print_step_ftbs) < dt/2.0D0 ) then
      call write_solution(nchandlers,time,ynp1)
    end if
    yn = ynp1
  end do
  call close_outfile(nchandlers)

  write(output_unit,*) 'SL Cubic Mean Error             : ', &
          sum(abs(ynp1-y0))/dble(nj)

  contains

    subroutine xaxis(x,x0,dx)
      implicit none
      real(8) , dimension(:) , intent(out) :: x
      real(8) , intent(in) :: x0 , dx
      integer :: j , nj
      nj = size(x)
      do j = 1 , nj
        x(j) = x0 + dble(j-1) * dx
      end do
    end subroutine xaxis

    subroutine phi(x,y,ishape)
      implicit none
      real(8) , dimension(:) , intent(in) :: x
      real(8) , dimension(:) , intent(out) :: y
      integer , intent(in) :: ishape

      y = 0.0D0
      if ( ishape == 1 ) then
        where ( 400.0D0 <= x .and. x <=  500.0D0 )
          y = 0.1D0 * (x - 400.0D0)
        end where
        where ( 500.0D0 < x .and. x <= 600.0D0 )
          y = 20.0D0 - 0.1D0 * (x - 400.0D0)
        end where
      else
        where ( 475.0D0 <= x .and. x <= 525.0D0 )
          y = 10.0D0
        end where
      end if
    end subroutine phi

    subroutine FTFS(ynp1,yn,u,dx,dt)
      implicit none
      real(8) , intent(in) :: u , dx , dt
      real(8) , dimension(:) , intent(in) :: yn
      real(8) , dimension(:) , intent(out) :: ynp1
      integer :: j , nj
      nj = size(ynp1)
      do j = 1 , nj-1
        ynp1(j) = yn(j) - u*dt/dx*(yn(j+1)-yn(j))
      end do
      !ynp1(nj) = yn(1) - u*dt/dx*(yn(2)-yn(1))
      ynp1(nj) = ynp1(1)
    end subroutine FTFS

    subroutine FTBS(ynp1,yn,u,dx,dt)
      implicit none
      real(8) , intent(in) :: u , dx , dt
      real(8) , dimension(:) , intent(in) :: yn
      real(8) , dimension(:) , intent(out) :: ynp1
      integer :: j , nj
      nj = size(ynp1)
      do j = 2 , nj
        ynp1(j) = yn(j) - u*dt/dx*(yn(j)-yn(j-1))
      end do
      !ynp1(1) = yn(nj) - u*dt/dx*(yn(nj)-yn(nj-1))
      ynp1(1) = ynp1(nj)
    end subroutine FTBS

    subroutine CTCS(ynp1,yn,ynm1,u,dx,dt)
      implicit none
      real(8) , intent(in) :: u , dx , dt
      real(8) , dimension(:) , intent(in) :: yn
      real(8) , dimension(:) , intent(in) :: ynm1
      real(8) , dimension(:) , intent(out) :: ynp1
      integer :: j , nj
      nj = size(ynp1)
      do j = 2 , nj-1
        ynp1(j) = ynm1(j) - u*dt/dx*(yn(j+1)-yn(j-1))
      end do
      ynp1(nj) = ynm1(nj) - u*dt/dx*(yn(2)-yn(nj-1))
      ynp1(1) = ynp1(nj)
    end subroutine CTCS

    subroutine RA_filter_update(alpha,yn,ynm1,ynp1,d)
      implicit none
      real(8) , intent(in) :: alpha
      real(8) , dimension(:) , intent(in) :: yn
      real(8) , dimension(:) , intent(in) :: ynm1
      real(8) , dimension(:) , intent(in) :: ynp1
      real(8) , dimension(:) , intent(out) :: d
      d = alpha * (ynm1 - 2.0D0 * yn + ynp1)
    end subroutine RA_filter_update

    subroutine SLAD_LIN(x,ynp1,yn,u,dx,dt)
      implicit none
      real(8) , intent(in) :: u , dx , dt
      real(8) , dimension(:) , intent(in) :: x
      real(8) , dimension(:) , intent(in) :: yn
      real(8) , dimension(:) , intent(out) :: ynp1
      real(8) :: xdep , alpha , deltax
      integer :: j , jm , nj
      nj = size(ynp1)
      deltax = abs(x(nj)-x(1))
      do j = 2 , nj
        xdep = x(j) - u*dt
        if ( xdep > x(nj) ) xdep = xdep - deltax
        if ( xdep < x(1)  ) xdep = xdep + deltax
        jm = int(ceiling(xdep/dx))+1
        alpha = (x(jm)-xdep)/dx
        ynp1(j) = yn(jm-1) * alpha + yn(jm) * ( 1.0D0-alpha )
      end do
      ynp1(1) = ynp1(nj)
    end subroutine SLAD_LIN

    subroutine SLAD_CUB(x,ynp1,yn,u,dx,dt)
      implicit none
      real(8) , intent(in) :: u , dx , dt
      real(8) , dimension(:) , intent(in) :: x
      real(8) , dimension(:) , intent(in) :: yn
      real(8) , dimension(:) , intent(out) :: ynp1
      real(8) :: xdep , alpha , deltax , c1 , c2 , c3 , c4
      integer :: j , nj , jmm2 , jmm1 , jm , jmp1
      nj = size(ynp1)
      deltax = abs(x(nj)-x(1))
      do j = 2 , nj
        xdep = x(j) - u*dt
        if ( xdep > x(nj) ) xdep = xdep - deltax
        if ( xdep < x(1)  ) xdep = xdep + deltax
        jm = int(ceiling(xdep/dx))+1
        alpha = (x(jm)-xdep)/dx
        c1 = - (alpha*(1.0D0-alpha*alpha))/6.0D0
        c2 = (alpha*(1.0D0+alpha)*(2.0D0-alpha))/2.0D0
        c3 = ((1.0D0-alpha*alpha)*(2.0D0-alpha))/2.0D0
        c4 = - (alpha*(1.0D0-alpha)*(2.0D0-alpha))/6.0D0
        jmm2 = jm - 2
        jmm1 = jm - 1
        jmp1 = jm + 1
        if ( jmm2 < 1 ) jmm2 = jmm2 + nj
        if ( jmm1 < 1 ) jmm1 = jmm1 + nj
        if ( jmp1 > nj ) jmp1 = jmp1 - nj
        ynp1(j) = c1*yn(jmm2) + c2*yn(jmm1) + c3*yn(jm) + c4*yn(jmp1)
      end do
      ynp1(1) = ynp1(nj)
    end subroutine SLAD_CUB

    subroutine open_outfile(pname,u,x,ncx,atype,aparam_alpha,aparam_beta)
      implicit none
      character(len=*) , intent(in) :: pname
      real(8) , intent(in) :: u
      real(8) , dimension(:) , intent(in) :: x
      integer , dimension(5) , intent(out) :: ncx
      character(len=*) , intent(in) :: atype
      real(8) , intent(in) ,optional :: aparam_alpha
      real(8) , intent(in) ,optional :: aparam_beta
      integer :: nx
      integer :: ires
      integer , dimension(2) :: idims
      nx = size(x)
      ires = nf90_create(trim(pname)//'.nc',NF90_CLOBBER,ncx(1))
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) 'Cannot create file ',trim(pname)//'.nc'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_put_att(ncx(1), nf90_global, 'title', atype)
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_put_att(ncx(1), nf90_global, 'comment', &
        'Cyclic boundary conditions')
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_put_att(ncx(1), nf90_global, 'velocity_in_m_s', u)
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      if ( present(aparam_alpha) ) then
        ires = nf90_put_att(ncx(1), nf90_global, 'alpha', aparam_alpha)
        if ( ires /= NF90_NOERR ) then
          write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
          write(error_unit,*) nf90_strerror(ires)
          stop
        end if
      end if
      if ( present(aparam_beta) ) then
        ires = nf90_put_att(ncx(1), nf90_global, 'beta', aparam_beta)
        if ( ires /= NF90_NOERR ) then
          write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
          write(error_unit,*) nf90_strerror(ires)
          stop
        end if
      end if
      ires = nf90_def_dim(ncx(1),'x',nx,idims(1)) 
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_def_dim(ncx(1),'time',NF90_UNLIMITED,idims(2)) 
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_def_var(ncx(1),'time',nf90_double,idims(2:2),ncx(2))
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_put_att(ncx(1), ncx(2), 'units', &
                          'seconds since 2000-01-01 00:00:00')
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_def_var(ncx(1),'x',nf90_double,idims(1:1),ncx(3))
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_put_att(ncx(1), ncx(3), 'units', 'm')
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_def_var(ncx(1),'y',nf90_double,idims,ncx(4))
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_put_att(ncx(1), ncx(4), 'units', 'm')
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_enddef(ncx(1))
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_put_var(ncx(1),ncx(3),x)
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ncx(5) = 1
    end subroutine open_outfile

    subroutine close_outfile(ncx)
      implicit none
      integer , dimension(5) , intent(inout) :: ncx
      integer :: ires
      ires = nf90_close(ncx(1))
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ncx = -1
    end subroutine close_outfile

    subroutine write_solution(ncx,time,y)
      implicit none
      integer , dimension(5) , intent(inout) :: ncx
      real(8) , intent(in) :: time
      real(8) , dimension(:) , intent(in) :: y
      integer , dimension(2) :: istart , icount
      real(8) , dimension(1) :: xtime
      integer :: ires
      istart(1) = 1
      icount(1) = size(y)
      istart(2) = ncx(5)
      icount(2) = 1
      xtime(1) = time
      ires = nf90_put_var(ncx(1),ncx(2),xtime,istart(2:2),icount(2:2))
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ires = nf90_put_var(ncx(1),ncx(4),y,istart,icount)
      if ( ires /= NF90_NOERR ) then
        write(error_unit,*) __FILE__, ' at line ',__LINE__,' : netcdf error'
        write(error_unit,*) nf90_strerror(ires)
        stop
      end if
      ncx(5) = ncx(5) + 1
    end subroutine write_solution

end program numII_advection
