#!/usr/bin/env python2
"""
Plot netcdf files produced by the numII_advection program.
The plot is created overplotting the solution at fixed points in space
(see the plot_spaces down).
The times at which the solution travels up to the requested points
is computed in the code given the velocity attribute in the input files.
If the scheme is unstable and the solution blows up, all the present
timesteps are included in the plot.
Title is built using the file global attributes.
"""

def plot_file(scheme,pp,advance=True):
  """

This function is doing the plot.
It requires as arguments the numII_advection scheme used and a pdf
handler to attach the figures as pages to.

Example usage:

from matplotlib.backends.backend_pdf import PdfPages
pp = PdfPages('numII_advection.pdf')
plot_file('CTCS',pp)

If the advance optional argument is set to False, no new figure
is produced

  """
  import numpy
  import pylab
  from scipy.io.netcdf import netcdf_file

  plot_spaces = numpy.array([ 0, 200, 600, 1000, 1500, 2000 ],numpy.float64)
  ncf = netcdf_file('numII_advection_'+scheme+'.nc')

  u = ncf.velocity_in_m_s
  title = ncf.title + '\nwith u = ' + str(u) + ' m/s'
  try:
    title = title + ', apha = ' + str(ncf.alpha)
  except:
    pass
  try:
    title = title + ', beta = ' + str(ncf.beta)
  except:
    pass

  pylab.title(title)
  pylab.xlabel('x ('+ncf.variables['x'].units+')')
  pylab.ylabel('y ('+ncf.variables['y'].units+')')

  x = ncf.variables['x'][:]
  times = numpy.array(ncf.variables['time'][:],numpy.int32)
  plot_times = numpy.array(plot_spaces/u,numpy.int32)
  try:
    idx = numpy.searchsorted(times,plot_times)
    y = ncf.variables['y'][idx,:]
  except:
    idx = range(0,len(times))
    y = ncf.variables['y'][idx,:]

  ncf.close( )

  for i in range(len(idx)):
    line = pylab.plot(x,y[i,:],label='t_0 + '+str(times[i])+' s')
  pylab.legend(loc=5,prop={'size':8})
  pylab.draw( )
  pylab.savefig(pp, format='pdf')
  if ( advance ):
    pylab.figure( )

if ( __name__ == '__main__'):
  from matplotlib.backends.backend_pdf import PdfPages
  schemes = ( 'FTFS', 'FTBS', 'CTCS', 'CTCS_RA', 'CTCS_RAW', 'SLLN', 'SLCB' )
  pp = PdfPages('numII_advection.pdf')
  for scheme in schemes:
    if ( scheme == schemes[-1] ):
      plot_file(scheme,pp,False)
    else:
      plot_file(scheme,pp)
  pp.close( )
