#!/usr/bin/env python2

"""
Small python function to plot the mean temperature from a RegCM output
file in a matplotlib plot window. The cartopy will download and cache
Natural Earth data required for vectorial overlays.

"""

def make_the_plot(filename):
  """
This function is computing time mean of the tas variable in the input file
and it is plotting the map on a matplotlib window.
  """

  from netCDF4 import num2date, date2num , Dataset
  import numpy as np
  import matplotlib.pyplot as plt
  import cartopy.crs as ccrs
  import cartopy.feature as cfeature

  ncf = Dataset(filename)
  projection = ccrs.PlateCarree( )
  xlat = ncf.variables['xlat'][:,:]
  xlon = ncf.variables['xlon'][:,:]
  tas = ncf.variables['tas'][:,0,:,:]
  times = ncf.variables['time']
  dates = num2date(times[:],units=times.units,calendar=times.calendar)

  tasmean = np.mean(tas,axis=0,keepdims=False)

  ax = plt.axes(projection=projection)

  ax.coastlines('10m')
  countries = cfeature.NaturalEarthFeature(
          scale='110m', category='cultural', name='admin_0_countries')
  ax.add_feature(countries, color='b', alpha=0.1)

  #places = cfeature.NaturalEarthFeature(
  #        scale='110m', category='cultural', name='populated_places')
  #ax.add_feature(places, color='b', hatch='o')

  ax.contourf(xlon,xlat,tasmean,transform=ccrs.PlateCarree(),cmap='spectral')

  # To mask out OCEAN or LAND
  #ax.add_feature(cfeature.OCEAN)
  #ax.add_feature(cfeature.LAND)

  ax.gridlines(crs=ccrs.PlateCarree(), draw_labels=True,
               linewidth=1, color='blue', alpha=0.5, linestyle='-')

  plt.title(ncf.title + '\n' + ncf.variables['tas'].long_name +
              ' [' + ncf.variables['tas'].units + ']\n' +
           'mean between ' + str(dates[0]) + ' and ' + str(dates[-1]) + '\n')

  plt.show()

if ( __name__ == '__main__' ):
  make_the_plot('../data/verysmall_SRF.1990010100.nc')
