#!/usr/bin/env python

"""
This small program is computing time means of netcdf file variables
of the RegCM output file and writing a netcdf file with the means.
"""

def compute_mean(datafile):
  """
Make the mean of an input regcm data output file.
Output file will be created in the current directory, and its name 
will follow the following convention:

    filename.nc  --> filename_mean.nc

Its format will be NETCDF4_CLASSIC, and all 2D+ variables will be
compressed in disk.
  """
  from netCDF4 import Dataset
  import numpy as np
  import time
  import os
  import re

  ncf = Dataset(datafile)
  nco = Dataset(os.path.basename(os.path.splitext(datafile)[0])
                +'_mean.nc', 'w', format='NETCDF4_CLASSIC')

  for attr in ncf.ncattrs():
    nco.setncattr(attr,getattr(ncf,attr))

  for dim in ncf.dimensions:
    if ( ncf.dimensions[dim].isunlimited() ):
      nco.createDimension(dim)
    else:
      nco.createDimension(dim,len(ncf.dimensions[dim]))

  for var in ncf.variables:
    nctype = ncf.variables[var].datatype
    if 'jx' in ncf.variables[var].dimensions:
      nco.createVariable(var,nctype,ncf.variables[var].dimensions,
                         zlib=True,complevel=9)
    else:
      nco.createVariable(var,nctype,ncf.variables[var].dimensions)
    if 'time' in ncf.variables[var].dimensions:
      nco.variables[var].setncattr('cell_methods','time: mean')
    for attr in ncf.variables[var].ncattrs():
      if attr is not 'cell_methods':
        nco.variables[var].setncattr(attr,getattr(ncf.variables[var],attr))

  nco.variables['time'][:] = np.mean(ncf.variables['time'][:])

  for var in ncf.variables:
    if 'time' not in ncf.variables[var].dimensions:
      nco.variables[var][:] = ncf.variables[var][:]
    else:
      if var is not 'time':
        dims = list(np.shape(ncf.variables[var]))
        time_axis = ncf.variables[var].dimensions.index('time')
        dims[time_axis] = 1
        # Why keepdims is not respected ? This SHOULD NOT be like this:
        nco.variables[var][:] = np.reshape(np.nanmean(ncf.variables[var][:],
              axis=time_axis,keepdims=True),tuple(dims))
        # SHOULD BE like this if keepdims was respected:
        #nco.variables[var][:] = np.nanmean(ncf.variables[var][:],
        #      axis=time_axis,keepdims=True)

  ncf.close()
  nco.close()

if ( __name__ == '__main__' ):
  compute_mean('../data/verysmall_SRF.1990010100.nc')
